==> unicore-uvos-server-1.5.1-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/db/InitDB.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.db;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.sql.SQLException;
import java.util.Properties;

import org.apache.ibatis.jdbc.RuntimeSqlException;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;

import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.engine.PersistentIdManager;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 * @author K. Benedyczak
 */
public class InitDB
{
	private static Properties dbProp;
	private static ComponentContainer container;
	
	private static void initDB(ScriptRunner runner) throws IOException, SQLException
	{
		System.out.println("Initialazing DB");
		String script = dbProp.getProperty(DB.P_CREATEDB_SCRIPT);
		if (script == null)
			script = ""; //yep - no default for this
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
	}

	private static void initData(ScriptRunner runner, String dbConf) throws IOException, SQLException
	{
		System.out.println("Putting initial data into DB");
		String script = dbProp.getProperty(DB.P_INIT_DATA_SCRIPT);
		if (script == null)
			script = DB.INIT_DATA_SCRIPT_DEF;
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
		try
		{
			PersistentIdManager idMan = container.getComponent(PersistentIdManager.class);
			idMan.regenerateAll();
		} catch (InternalException e)
		{
			e.printStackTrace();
			throw new IOException(e.toString());
		}
	}
	
	private static void checkDB(ScriptRunner runner) throws IOException, RuntimeSqlException
	{
		System.out.println("Checking content of DB");
		String script = dbProp.getProperty(DB.P_CHECK_SCRIPT);
		if (script == null)
			script = DB.CHECK_SCRIPT_DEF;
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
	}

	private static void clearDB(ScriptRunner runner) throws IOException, SQLException
	{
		System.out.println("Clearing DB");
		String script = dbProp.getProperty(DB.P_CLEAR_SCRIPT);
		if (script == null)
			script = DB.CLEAR_SCRIPT_DEF;
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
	}
	
	
	public static void work(String[] args) throws FileNotFoundException, IOException, SQLException
	{
		String configFile = VOServer.CONFIGURATION_FILE;
		if (args.length > 0)
			configFile = args[0];
		boolean batchMode = args.length > 1 && args[1].equals("-b");

		container = new ComponentContainer(configFile);
		Configuration mainConf = container.getComponent(Configuration.class);
		String dbProperties = mainConf.getAsString(
				ConfDefaults.P_DBPROPERTIES_FILE, 
				ConfDefaults.DBPROPERTIES_FILE_DEF, null);
		String dbConf = mainConf.getAsString(ConfDefaults.P_DBCONFIG_FILE, 
				ConfDefaults.DBCONFIG_FILE_DEF, null);
		dbProp = new Properties();
		dbProp.load(new FileInputStream(dbProperties));
		
		Reader reader = new FileReader(dbConf);
		SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();
		SqlSessionFactory factory = builder.build(reader);

		SqlSession session = factory.openSession();
		SqlSession session1 = factory.openSession();
		
		ScriptRunner runner = new ScriptRunner(session.getConnection());
		runner.setAutoCommit(true);
		runner.setStopOnError(true);
		
		ScriptRunner runner1 = new ScriptRunner(session1.getConnection());
		runner1.setAutoCommit(true);
		runner1.setStopOnError(false);

		if (batchMode)
		{
			PrintWriter nullPrinter = new PrintWriter(new NullWriter());
			runner.setErrorLogWriter(nullPrinter);
			runner.setLogWriter(nullPrinter);
			runner1.setErrorLogWriter(nullPrinter);
			runner1.setLogWriter(nullPrinter);
		}
		try
		{
			checkDB(runner);
		} catch (RuntimeSqlException e)
		{
			if (!batchMode && !askQuestion())
				return;
		}
		clearDB(runner1);
		
		String driver = dbProp.getProperty("driver");
		if (driver.equals("com.mysql.jdbc.Driver"))
			runner.setDelimiter("#");
		initDB(runner);
		runner.setDelimiter(";");
		
		initData(runner, dbConf);
		if ("org.hsqldb.jdbcDriver".equals(driver))
		{
			String cmd = "SHUTDOWN;";
			StringReader sr = new StringReader(cmd); 
			runner.runScript(sr);
			sr.close();
		}
		
		session.close();
	}
	
	public static void main(String[] args)
	{
		try
		{
			work(args);
		} catch (Exception e)
		{
			e.printStackTrace();
		}
	}
		
	private static boolean askQuestion()
	{
		System.out.println("\n\nIt seems that your database " +
				"is initialized.\n" +
				"Proceeding will REMOVE the whole " +
				"actual content.\n\n" +
				"ARE YOU SURE? [N]");
		BufferedReader r = new BufferedReader(new InputStreamReader(System.in));
		String answer;
		try
		{
			answer = r.readLine();
			if (answer!=null && answer.equalsIgnoreCase("y"))
				return true;
		} catch (IOException e){}
		return false;
	}
	
	private static class NullWriter extends Writer
	{
		@Override
		public void close() throws IOException
		{
		}
		@Override
		public void flush() throws IOException
		{
		}
		@Override
		public void write(char[] cbuf, int off, int len) throws IOException
		{
		}
	}
}

==> unicore-uvos-server-1.6.0-0.src.tar.gz: src/main/java/pl/edu/icm/unicore/uvos/db/InitDB.java <==
/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 17, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.db;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.sql.SQLException;
import java.util.Properties;

import org.apache.ibatis.jdbc.RuntimeSqlException;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;

import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.engine.PersistentIdManager;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.UVOSServerProperties;

/**
 * @author K. Benedyczak
 */
public class InitDB
{
	private static Properties dbProp;
	private static ComponentContainer container;
	
	private static void initDB(ScriptRunner runner) throws IOException, SQLException
	{
		System.out.println("Initialazing DB");
		String script = dbProp.getProperty(DB.P_CREATEDB_SCRIPT);
		if (script == null)
			script = ""; //yep - no default for this
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
	}

	private static void initData(ScriptRunner runner, String dbConf) throws IOException, SQLException
	{
		System.out.println("Putting initial data into DB");
		String script = dbProp.getProperty(DB.P_INIT_DATA_SCRIPT);
		if (script == null)
			script = DB.INIT_DATA_SCRIPT_DEF;
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
		try
		{
			PersistentIdManager idMan = container.getComponent(PersistentIdManager.class);
			idMan.regenerateAll();
		} catch (InternalException e)
		{
			e.printStackTrace();
			throw new IOException(e.toString());
		}
	}
	
	private static void checkDB(ScriptRunner runner) throws IOException, RuntimeSqlException
	{
		System.out.println("Checking content of DB");
		String script = dbProp.getProperty(DB.P_CHECK_SCRIPT);
		if (script == null)
			script = DB.CHECK_SCRIPT_DEF;
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
	}

	private static void clearDB(ScriptRunner runner) throws IOException, SQLException
	{
		System.out.println("Clearing DB");
		String script = dbProp.getProperty(DB.P_CLEAR_SCRIPT);
		if (script == null)
			script = DB.CLEAR_SCRIPT_DEF;
		FileReader reader=new FileReader(script);
		try{
			runner.runScript(reader);
		}
		finally{
			reader.close();
		}
	}
	
	
	public static void work(String[] args) throws FileNotFoundException, IOException, SQLException
	{
		String configFile = VOServer.CONFIGURATION_FILE;
		if (args.length > 0)
			configFile = args[0];
		boolean batchMode = args.length > 1 && args[1].equals("-b");

		container = new ComponentContainer(configFile);
		UVOSServerProperties mainConf = container.getComponent(UVOSServerProperties.class);
		String dbProperties = mainConf.getValue(UVOSServerProperties.DBPROPERTIES_FILE);
		String dbConf = mainConf.getValue(UVOSServerProperties.DBCONFIG_FILE);
		dbProp = new Properties();
		dbProp.load(new FileInputStream(dbProperties));
		
		Reader reader = new FileReader(dbConf);
		SqlSessionFactoryBuilder builder = new SqlSessionFactoryBuilder();
		SqlSessionFactory factory = builder.build(reader);

		SqlSession session = factory.openSession();
		SqlSession session1 = factory.openSession();
		
		ScriptRunner runner = new ScriptRunner(session.getConnection());
		runner.setAutoCommit(true);
		runner.setStopOnError(true);
		
		ScriptRunner runner1 = new ScriptRunner(session1.getConnection());
		runner1.setAutoCommit(true);
		runner1.setStopOnError(false);

		if (batchMode)
		{
			PrintWriter nullPrinter = new PrintWriter(new NullWriter());
			runner.setErrorLogWriter(nullPrinter);
			runner.setLogWriter(nullPrinter);
			runner1.setErrorLogWriter(nullPrinter);
			runner1.setLogWriter(nullPrinter);
		}
		try
		{
			checkDB(runner);
		} catch (RuntimeSqlException e)
		{
			if (!batchMode && !askQuestion())
				return;
		}
		clearDB(runner1);
		
		String driver = dbProp.getProperty("driver");
		if (driver.equals("com.mysql.jdbc.Driver"))
			runner.setDelimiter("#");
		initDB(runner);
		runner.setDelimiter(";");
		
		initData(runner, dbConf);
		if ("org.hsqldb.jdbcDriver".equals(driver))
		{
			String cmd = "SHUTDOWN;";
			StringReader sr = new StringReader(cmd); 
			runner.runScript(sr);
			sr.close();
		}
		
		session.close();
	}
	
	public static void main(String[] args)
	{
		try
		{
			work(args);
		} catch (Exception e)
		{
			e.printStackTrace();
		}
	}
		
	private static boolean askQuestion()
	{
		System.out.println("\n\nIt seems that your database " +
				"is initialized.\n" +
				"Proceeding will REMOVE the whole " +
				"actual content.\n\n" +
				"ARE YOU SURE? [N]");
		BufferedReader r = new BufferedReader(new InputStreamReader(System.in));
		String answer;
		try
		{
			answer = r.readLine();
			if (answer!=null && answer.equalsIgnoreCase("y"))
				return true;
		} catch (IOException e){}
		return false;
	}
	
	private static class NullWriter extends Writer
	{
		@Override
		public void close() throws IOException
		{
		}
		@Override
		public void flush() throws IOException
		{
		}
		@Override
		public void write(char[] cbuf, int off, int len) throws IOException
		{
		}
	}
}
